#ifndef CALCULATOR_DEMO_OBJECTS
#define CALCULATOR_DEMO_OBJECTS

#include "FT810_Types.h"

typedef enum {taNone, taLeft, taCenter, taRight, taCenterX, taCenterY, taRightX} TTextAlign;

typedef struct Screen TScreen;

typedef unsigned long TPointer;

typedef struct tagObjInfo {
  TPointer Obj;
  char     Type;
  char     Order;
  char     Flags;

  char     HitTag;
  int      HitX;
  int      HitY;
} TObjInfo;

typedef struct tagTouchStat {
  char Pressed;

  char Tag;
  int  X;
  int  Y;

  TObjInfo ActObjInfo;
} TTouchStat;

typedef void (*TDrawHandler)(TPointer aObj);

typedef void (*TEvtAction)();

typedef struct tagEvtSound {
  char SndAct;
  char Effect;
  char Pitch;
  char Volume;
} TEvtSound;

typedef const struct tagCEvent {
  TEvtAction Action;
  TEvtSound  Sound;
} TCEvent;

typedef struct tagEvent {
  TEvtAction Action;
  TEvtSound  Sound;
} TEvent;

typedef const struct tagCRect {
  int Left;
  int Top;
  int Width;
  int Height;
} TCRect;

typedef struct tagRect {
  int Left;
  int Top;
  int Width;
  int Height;
} TRect;

typedef const code struct tagCButton {
  TScreen         *OwnerScreen;
  char            Order;
  char            Visible;
  char            Opacity;
  char            Tag;
  int             Left;
  int             Top;
  int             Width;
  int             Height;
  char            Pen_Width;
  unsigned int    Pen_Color;
  unsigned int    Color;
  unsigned int    Press_Color;
  unsigned int    ColorTo;
  unsigned int    Press_ColorTo;
  char            Gradient;
  const code char *Caption;
  TTextAlign      TextAlign;
  const code char *FontName;
  unsigned int    Font_Color;
  char            FontHandle;
  long            Source;
  char            Active;
  TCEvent         *OnUp;
  TCEvent         *OnDown;
  TCEvent         *OnClick;
  TCEvent         *OnPress;
} TCButton;

typedef const code struct tagCButton_Round {
  TScreen         *OwnerScreen;
  char            Order;
  char            Visible;
  char            Opacity;
  char            Tag;
  int             Left;
  int             Top;
  int             Width;
  int             Height;
  char            Pen_Width;
  unsigned int    Pen_Color;
  unsigned int    Color;
  unsigned int    Press_Color;
  const code char *Caption;
  TTextAlign      TextAlign;
  const code char *FontName;
  unsigned int    Font_Color;
  char            FontHandle;
  long            Source;
  char            Corner_Radius;
  char            Active;
  TCEvent         *OnUp;
  TCEvent         *OnDown;
  TCEvent         *OnClick;
  TCEvent         *OnPress;
} TCButton_Round;

typedef struct tagLabel {
  TScreen         *OwnerScreen;
  char            Order;
  char            Visible;
  char            Opacity;
  char            Tag;
  int             Left;
  int             Top;
  int             Width;
  int             Height;
  char            *Caption;
  const code char *FontName;
  unsigned int    Font_Color;
  char            FontHandle;
  long            Source;
  char            Active;
  TEvent          *OnUp;
  TEvent          *OnDown;
  TEvent          *OnClick;
  TEvent          *OnPress;
} TLabel;

typedef const code struct tagCLabel {
  TScreen         *OwnerScreen;
  char            Order;
  char            Visible;
  char            Opacity;
  char            Tag;
  int             Left;
  int             Top;
  int             Width;
  int             Height;
  const code char *Caption;
  const code char *FontName;
  unsigned int    Font_Color;
  char            FontHandle;
  long            Source;
  char            Active;
  TCEvent         *OnUp;
  TCEvent         *OnDown;
  TCEvent         *OnClick;
  TCEvent         *OnPress;
} TCLabel;

typedef const code struct tagCBox {
  TScreen         *OwnerScreen;
  char            Order;
  char            Visible;
  char            Opacity;
  char            Tag;
  int             Left;
  int             Top;
  int             Width;
  int             Height;
  char            Pen_Width;
  unsigned int    Pen_Color;
  unsigned int    Color;
  unsigned int    Press_Color;
  unsigned int    ColorTo;
  unsigned int    Press_ColorTo;
  char            Gradient;
  char            Active;
  TCEvent         *OnUp;
  TCEvent         *OnDown;
  TCEvent         *OnClick;
  TCEvent         *OnPress;
} TCBox;

struct Screen {
  unsigned int   Color;
  unsigned int   Width;
  unsigned int   Height;
  unsigned short ObjectsCount;
  unsigned short CButtonsCount;
  TCButton       *const code *CButtons;
  unsigned short CButtons_RoundCount;
  TCButton_Round *const code *CButtons_Round;
  unsigned short LabelsCount;
  TLabel         *const code *Labels;
  unsigned short CLabelsCount;
  TCLabel        *const code *CLabels;
  unsigned short CBoxesCount;
  TCBox          *const code *CBoxes;
  unsigned long  DynResStart;
  unsigned short Active;
  unsigned short SniffObjectEvents;
  TEvent         *OnUp;
  TEvent         *OnDown;
  TEvent         *OnTagChange;
  TEvent         *OnPress;
};


// Object type constants
// Usage: VTFT stack internally
extern const VTFT_OT_CBUTTON;
extern const VTFT_OT_CBUTTONROUND;
extern const VTFT_OT_LABEL;
extern const VTFT_OT_CLABEL;
extern const VTFT_OT_CBOX;
// ~Object type constants

// Event type constants
// Usage: OnEvent
extern const VTFT_EVT_UP;
extern const VTFT_EVT_DOWN;
extern const VTFT_EVT_CLICK;
extern const VTFT_EVT_PRESS;
// ~Event type constants

// Sound action constants
// Usage: sound event action property and ProcessEvent
extern const VTFT_SNDACT_NONE;
extern const VTFT_SNDACT_PLAY;
extern const VTFT_SNDACT_STOP;
// ~Sound action constants

// Resource loading constants.
// Usage: DrawScreenO and LoadCurrentScreenResToGRAM
extern const VTFT_LOAD_RES_NONE;
extern const VTFT_LOAD_RES_STATIC;
extern const VTFT_LOAD_RES_DYNAMIC;
extern const VTFT_LOAD_RES_ALL;
// ~Resource loading constants

// Display effect constants
// Usage: DrawScreenO
extern const VTFT_DISPLAY_EFF_NONE;
extern const VTFT_DISPLAY_EFF_LIGHTS_FADE;
extern const VTFT_DISPLAY_EFF_LIGHTS_OFF;
// ~Display effect constants

// Stack flags
// Usage: internally used by VTFT stack
extern const VTFT_INT_REPAINT_ON_DOWN;
extern const VTFT_INT_REPAINT_ON_UP;
extern const VTFT_INT_BRING_TO_FRONT;
extern const VTFT_INT_INTRINSIC_CLICK_EFF;
// ~Stack flags

// Table of object draw handlers
// Use object type constants to access coresponding object draw handler
extern const TPointer DrawHandlerTable[44];
// ~Table of draw handler pointers


// Default configuration parameters
extern const TFT810PWM            VTFT_FT810_CONFIG_PWM;
extern const TFT810GPIO           VTFT_FT810_CONFIG_GPIO;
extern const TFT810Sound          VTFT_FT810_CONFIG_SOUND;
extern const TFT810Audio          VTFT_FT810_CONFIG_AUDIO;
extern const TFT810Display        VTFT_FT810_CONFIG_DISPLAY;
extern const TFT810Interrupt      VTFT_FT810_CONFIG_INTERRUPT;
extern const TFT810Touch          VTFT_FT810_CONFIG_TOUCH;

extern TTouchStat TouchS;

extern TScreen Calc_Demo;

extern TCButton       Button1;
extern TCBox          Box1;
extern TCButton       Button2;
extern TCEvent        Button2_OnClick;
extern TCButton       Button3;
extern TCEvent        Button3_OnClick;
extern TCButton       Button4;
extern TCEvent        Button4_OnClick;
extern TCButton       Button5;
extern TCEvent        Button5_OnClick;
extern TCButton       Button6;
extern TCEvent        Button6_OnClick;
extern TCButton       Button7;
extern TCEvent        Button7_OnClick;
extern TCButton       Button8;
extern TCEvent        Button8_OnClick;
extern TCButton       Button9;
extern TCEvent        Button9_OnClick;
extern TCButton       Button10;
extern TCEvent        Button10_OnClick;
extern TCButton       Button11;
extern TCEvent        Button11_OnClick;
extern TCButton       Button12;
extern TCEvent        Button12_OnClick;
extern TCButton       Button13;
extern TCEvent        Button13_OnClick;
extern TCButton       Button14;
extern TCEvent        Button14_OnClick;
extern TCButton       Button15;
extern TCEvent        Button15_OnClick;
extern TCButton       Button16;
extern TCEvent        Button16_OnClick;
extern TCButton       Button17;
extern TCEvent        Button17_OnClick;
extern TCButton       Button19;
extern TCEvent        Button19_OnClick;
extern TCButton       Button20;
extern TCEvent        Button20_OnClick;
extern TLabel         Label1;
extern TLabel         Label2;
extern TLabel         Label3;
extern TLabel         Label4;
extern TLabel         Label6;
extern TCButton_Round ButtonRound1;
extern TCEvent        ButtonRound1_OnClick;

extern TCButton       *const code Calc_Demo_CButtons[19];
extern TCButton_Round *const code Calc_Demo_CButtons_Round[1];
extern TLabel         *const code Calc_Demo_Labels[5];
extern TCBox          *const code Calc_Demo_CBoxes[1];

extern TScreen Main_Screen;

extern TCLabel  Label5;
extern TCLabel  Label7;
extern TCLabel  Label8;
extern TCLabel  Label9;
extern TCButton Button18;
extern TCEvent  Button18_OnClick;

extern TCButton *const code Main_Screen_CButtons[1];
extern TCLabel  *const code Main_Screen_CLabels[4];

extern TScreen *CurrentScreen;

/////////////////////////
// User Event Handlers
void Button10OnClick();
void Button11OnClick();
void Button12OnClick();
void Button13OnClick();
void Button14OnClick();
void Button15OnClick();
void Button16OnClick();
void Button17OnClick();
void Button18OnClick();
void Button19OnClick();
void Button20OnClick();
void Button2OnClick();
void Button3OnClick();
void Button4OnClick();
void Button5OnClick();
void Button6OnClick();
void Button7OnClick();
void Button8OnClick();
void Button9OnClick();
void ButtonRound1OnClick();
/////////////////////////

/////////////////////////////////
// Captions
extern const code char Button1_Caption[];
extern const code char Box1_Caption[];
extern const code char Button2_Caption[];
extern const code char Button3_Caption[];
extern const code char Button4_Caption[];
extern const code char Button5_Caption[];
extern const code char Button6_Caption[];
extern const code char Button7_Caption[];
extern const code char Button8_Caption[];
extern const code char Button9_Caption[];
extern const code char Button10_Caption[];
extern const code char Button11_Caption[];
extern const code char Button12_Caption[];
extern const code char Button13_Caption[];
extern const code char Button14_Caption[];
extern const code char Button15_Caption[];
extern const code char Button16_Caption[];
extern const code char Button17_Caption[];
extern const code char Button19_Caption[];
extern const code char Button20_Caption[];
extern char Label1_Caption[];
extern char Label2_Caption[];
extern char Label3_Caption[];
extern char Label4_Caption[];
extern char Label6_Caption[];
extern const code char ButtonRound1_Caption[];
extern const code char Label5_Caption[];
extern const code char Label7_Caption[];
extern const code char Label8_Caption[];
extern const code char Label9_Caption[];
extern const code char Button18_Caption[];

/////////////////////////////////
// Event Descriptors 
extern const code TEvent Button2_OnClick;
extern const code TEvent Button3_OnClick;
extern const code TEvent Button4_OnClick;
extern const code TEvent Button5_OnClick;
extern const code TEvent Button6_OnClick;
extern const code TEvent Button7_OnClick;
extern const code TEvent Button8_OnClick;
extern const code TEvent Button9_OnClick;
extern const code TEvent Button10_OnClick;
extern const code TEvent Button11_OnClick;
extern const code TEvent Button12_OnClick;
extern const code TEvent Button13_OnClick;
extern const code TEvent Button14_OnClick;
extern const code TEvent Button15_OnClick;
extern const code TEvent Button16_OnClick;
extern const code TEvent Button17_OnClick;
extern const code TEvent Button19_OnClick;
extern const code TEvent Button20_OnClick;
extern const code TEvent ButtonRound1_OnClick;
extern const code TEvent Button18_OnClick;
/////////////////////////////////

void DrawScreenO(TScreen *aScreen, char aOptions);
void DrawScreen(TScreen *aScreen);
void DrawCButton(TCButton *ACButton);
void DrawCButtonRound(TCButton_Round *ACButtonRound);
void DrawLabel(TLabel *ALabel);
void DrawCLabel(TCLabel *ACLabel);
void DrawCBox(TCBox *ACBox);
void ProcessVTFTStack();
void InitVTFTStack();

#endif
